from django.conf.global_settings import LANGUAGES as FULL_LANGUAGES_LIST
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericRelation
from django.db.models import Count, FileField
from django.forms.fields import CheckboxInput
from django.utils.translation import gettext_lazy as _
from django_filters import (
    BooleanFilter,
    CharFilter,
    ChoiceFilter,
    DateRangeFilter,
    FilterSet,
    ModelChoiceFilter,
)

from .models import Collection, Link, Tag

languages_dict = dict(FULL_LANGUAGES_LIST)


class LinkFilter(FilterSet):
    highlight = BooleanFilter(
        field_name="highlight", label=_("Highlighted"), widget=CheckboxInput
    )
    added_by = ModelChoiceFilter(
        field_name="added_by",
        label=_("Added by"),
        queryset=get_user_model().objects.all(),
    )
    date_added = DateRangeFilter(field_name="date_added", label=_("Date added"))
    language = ChoiceFilter(
        field_name="language",
        label=_("Language"),
        # choices=get_languages_list,  # moved in __init__!
    )
    online = BooleanFilter(field_name="online", label=_("Is online"))
    comments = BooleanFilter(field_name="comments", label=_("Has comments"))

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # get choices with queryset based on request parameter (view name/domain name if present)
        if "domain_name" in self.request.resolver_match.kwargs:
            self.filters["language"].extra["choices"] = self.get_languages_list(
                self.request.resolver_match.kwargs["domain_name"]
            )
        else:
            self.filters["language"].extra["choices"] = self.get_languages_list()

    def get_languages_list(self, domain_name=None, *args, **kwargs):
        if domain_name:
            languages = (
                Link.objects.filter(link__contains=domain_name)
                .values("language")
                .distinct()
                .annotate(total=Count("id"))
                .order_by("-total")
            )
        else:
            languages = (
                Link.objects.all()
                .values("language")
                .distinct()
                .annotate(total=Count("id"))
                .order_by("-total")
            )

        languages_choices = []
        for language in languages:
            if language["language"] in languages_dict:
                language_txt = f"{_(languages_dict[language['language']])} ({str(language['total'])})"
                languages_choices.append((language["language"], language_txt))
            elif language["language"] is None:
                languages_choices.append(
                    ("null", _("No language") + " (" + str(language["total"]) + ")")
                )

        return languages_choices

    class Meta:
        model = Link

        fields = [
            "highlight",
            "language",
            "online",
            "added_by",
            "date_added",
            "comments",
        ]

        exclude = ["allow_override_title", "allow_override_language"]

        filter_overrides = {
            # FileField: {
            #     "filter_class": BooleanFilter,
            #     "extra": lambda f: {
            #         "lookup_expr": "not__isnull",
            #     }
            # },
            GenericRelation: {
                "filter_class": BooleanFilter,
                "extra": lambda f: {
                    "lookup_expr": "not__isnull",
                },
            },
            FileField: {
                "filter_class": BooleanFilter,
                "extra": lambda f: {
                    "lookup_expr": "not__isnull",
                },
            },
        }


class CollectionFilter(FilterSet):
    highlight = BooleanFilter(
        field_name="highlight", label=_("Highlighted"), widget=CheckboxInput
    )
    added_by = ModelChoiceFilter(
        field_name="added_by",
        label=_("Added by"),
        queryset=get_user_model().objects.all(),
    )
    date_added = DateRangeFilter(field_name="date_added", label=_("Date added"))

    class Meta:
        model = Collection

        fields = [
            "highlight",
            "added_by",
            "date_added",
        ]


class TagsFilter(FilterSet):
    highlight = BooleanFilter(
        field_name="highlight", label=_("Highlighted"), widget=CheckboxInput
    )
    name = CharFilter(
        field_name="translations__tag", label=_("Tag name"), lookup_expr="icontains"
    )
    date_added = DateRangeFilter(field_name="date_added", label=_("Date added"))

    class Meta:
        model = Tag

        fields = [
            "name",
            "highlight",
            "date_added",
        ]



# tests/test_views.py

import pytest
from django.contrib.auth import get_user_model
from django.urls import reverse

from share_links.models import AboutContactPages, Category, Collection, Link, Tag


@pytest.mark.django_db
def test_search_view(client):
    Link.objects.create(link="https://example.com", title="Example Title")

    response = client.get(reverse("search"), {"search": "example"})
    assert response.status_code == 200
    assert "links" in response.context
    assert "tags" in response.context


@pytest.mark.django_db
def test_links_view(client):
    Link.objects.create(link="https://example.com", title="Example Title")

    response = client.get(reverse("homepage"))
    assert response.status_code == 200
    assert "object_list" in response.context


@pytest.mark.django_db
def test_link_view(client):
    link = Link.objects.create(link="https://example.com", title="Example Title")

    response = client.get(reverse("link", args=[link.pk]))
    assert response.status_code == 200
    assert "object" in response.context


@pytest.mark.django_db
def test_link_index_view(client):
    Link.objects.create(link="https://example.com", title="Example Title")

    response = client.get(reverse("link_index", kwargs={"domain_name": "example.com"}))
    assert response.status_code == 200
    assert "object_list" in response.context


@pytest.mark.django_db
def test_tags_view(client):
    Tag.objects.create(tag="Example Tag")

    response = client.get(reverse("tags"))
    assert response.status_code == 200
    assert "object_list" in response.context


@pytest.mark.django_db
def test_tag_view(client):
    tag = Tag.objects.create(tag="Example Tag")
    link = Link.objects.create(link="https://example.com", title="Example Title")
    link.tags.add(tag)

    response = client.get(reverse("tag", args=[tag.slug]))
    assert response.status_code == 200
    assert "object_list" in response.context


@pytest.mark.django_db
def test_category_view(client):
    category = Category.objects.create(label="Example Category")

    response = client.get(reverse("category", args=[category.pk]))
    assert response.status_code == 200
    assert "object" in response.context


@pytest.mark.django_db
def test_stats_view(client):
    Link.objects.create(link="https://example.com", title="Example Title")

    response = client.get(reverse("stats"))
    assert response.status_code == 200
    assert "stats" in response.context


@pytest.mark.django_db
def test_webring_view(client):
    response = client.get(reverse("webring"))
    assert response.status_code == 200
    assert "webring" in response.context


@pytest.mark.django_db
def test_get_title_view(client):
    User = get_user_model()
    user = User.objects.create_user(username="testuser", password="testpass")
    client.force_login(user)

    response = client.get(reverse("get_title"), {"url": "https://example.com"})
    assert response.status_code == 200
    assert "status" in response.json()
    assert "title" in response.json()


@pytest.mark.django_db
def test_get_lang_view(client):
    User = get_user_model()
    user = User.objects.create_user(username="testuser", password="testpass")
    client.force_login(user)

    response = client.get(reverse("get_lang"), {"url": "https://example.com"})
    assert response.status_code == 200
    assert "status" in response.json()
    assert "lang" in response.json()


@pytest.mark.django_db
def test_about_view(client):
    AboutContactPages.objects.create(page_title="About Page")

    response = client.get(reverse("about"))
    assert response.status_code == 200
    assert "about_text" in response.context


@pytest.mark.django_db
def test_contact_view(client):
    AboutContactPages.objects.create(page_title="Contact Page")

    response = client.get(reverse("contact"))
    assert response.status_code == 200
    assert "contact_text" in response.context


@pytest.mark.django_db
def test_rss_new_view(client):
    Link.objects.create(link="https://example.com", title="Example Title")

    response = client.get(reverse("rss_new"))
    assert response.status_code == 200


@pytest.mark.django_db
def test_rss_new_links_view(client):
    Link.objects.create(link="https://example.com", title="Example Title")

    response = client.get(reverse("rss_new_links"))
    assert response.status_code == 200


@pytest.mark.django_db
def test_rss_new_tags_view(client):
    Tag.objects.create(tag="Example Tag")

    response = client.get(reverse("rss_new_tags"))
    assert response.status_code == 200


@pytest.mark.django_db
def test_collections_view(client):
    Collection.objects.create(name="Example Collection")

    response = client.get(reverse("collections"))
    assert response.status_code == 200
    assert "object_list" in response.context


@pytest.mark.django_db
def test_collection_view(client):
    collection = Collection.objects.create(name="Example Collection")

    response = client.get(reverse("collection", args=[collection.slug]))
    assert response.status_code == 200
    assert "object_list" in response.context



from django.conf import settings
from django.core.management.base import BaseCommand

from share_links.models import Link
from share_links.utils import get_title

languages_list = settings.LANGUAGES


class Command(BaseCommand):
    help = "Loop on each saved link and add a title if it's not defined."

    def handle(self, *args, **options):
        nb = 0
        errors = []
        links = Link.objects.all()
        nb_links = len(links)
        for link in links:
            if link.allow_override_title:
                self.stdout.write(
                    f"[{str(nb + len(errors))}/{nb_links}] Adding title to {link.link} if missing..."
                )
                try:
                    title = get_title(link.link)
                    for language in languages_list:
                        self.stdout.write(f"Doing {language[0]}... ")
                        link.set_current_language(language[0])
                        try:
                            if link.title is None or link.title == "":
                                self.stdout.write(
                                    self.style.SUCCESS(f"  New title: {title}")
                                )
                                link.title = title
                                link.save()
                            else:
                                self.stdout.write(f"  No new title: '{link.title}'")
                        except link.DoesNotExist:
                            self.stdout.write(f"  New title: {title}")
                            link.title = title
                            link.save()
                    nb += 1
                    self.stdout.write("Done!")
                except Exception as e:
                    self.stdout.write(self.style.ERROR(f"Error: {e}"))
                    errors.append(link)
            else:
                f"[{str(nb + len(errors))}/{nb_links}] Do not update title for {link.link} (allow_override_title is False)..."
                nb += 1

        self.stdout.write("")
        self.stdout.write(self.style.SUCCESS(f"Added {nb} titles!"))

        self.stdout.write(self.style.ERROR(f"There was {str(len(errors))} errors:"))
        for error in errors:
            self.stdout.write(f"[{error.id}] {error.link}")



from json import load
from random import choice
from urllib.request import urlopen

from django.contrib.auth import get_user_model
from django.contrib.syndication.views import Feed
from django.db.models import Count, Q
from django.http import JsonResponse
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.html import escape, strip_tags
from django.utils.translation import activate, get_language
from django.utils.translation import gettext_lazy as _
from django.views.generic import DetailView, ListView
from django.views.generic.base import TemplateView
from django_filters.views import FilterView

from share_links.apps.share_links_comments.models import CommentwithCaptcha
from share_links.conf import PAGINATION_SIZE, SEARCH_MIN_LENGTH, SHOW_WEBARCHIVE_LINK

from .filters import CollectionFilter, LinkFilter, TagsFilter
from .forms import SearchForm
from .models import AboutContactPages, Category, Collection, Link, Tag
from .utils import get_lang, get_title


class SearchView(TemplateView):
    template_name = "share_links/search/results.html"
    context_object_name = "links_list"

    def get_context_data(self, **kwargs):
        activate(get_language())
        context = super().get_context_data(**kwargs)
        context["request_min_chars"] = SEARCH_MIN_LENGTH

        if self.request.method == "GET":
            form = SearchForm(self.request.GET)
            if (
                form.is_valid()
                and len(form.cleaned_data["search"])  # noqa
                >= SEARCH_MIN_LENGTH  # noqa
            ):
                context["links"] = self.get_links_list(
                    form.cleaned_data["search"].lower()
                )
                context["tags"] = self.get_tags_list(self.request.GET["search"].lower())
                return context

        return context

    def get_links_list(self, request):
        return Link.objects.filter(
            Q(link__icontains=request)  # noqa
            | Q(translations__title__icontains=request)  # noqa
            | Q(translations__description__icontains=request)  # noqa
            | Q(tags__translations__tag__icontains=request)  # noqa
        ).distinct()

    def get_tags_list(self, request):
        return Tag.objects.filter(
            Q(translations__tag__icontains=request)  # noqa
            | Q(translations__description__icontains=request)  # noqa
        ).distinct()


class LinksView(FilterView):
    model = Link
    paginate_by = PAGINATION_SIZE
    template_name = "share_links/link/list.html"
    filterset_class = LinkFilter
    queryset = (
        Link.objects.all()
        .order_by("-id")
        .prefetch_related(
            "tags",
            "translations",
            "tags__translations",
            "comments",
            "added_by",
        )
    )


class LinkView(DetailView):
    model = Link
    template_name = "share_links/link/detail.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context["show_webarchive_link"] = SHOW_WEBARCHIVE_LINK
        return context


class LinkIndexView(FilterView):
    model = Link
    paginate_by = PAGINATION_SIZE
    template_name = "share_links/link/list.html"
    filterset_class = LinkFilter

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["title"] = (
            _("All links for domain") + f" <i>{self.kwargs['domain_name']}</i>"
        )
        return context


def go_random_link(request):
    links = list(Link.objects.all())
    if len(links):
        return redirect(choice(links).link)
    else:
        return redirect("homepage")


class TagsView(FilterView):
    model = Tag
    paginate_by = PAGINATION_SIZE
    template_name = "share_links/tag/list.html"
    filterset_class = TagsFilter


class TagView(ListView):
    model = Link
    paginate_by = PAGINATION_SIZE
    template_name = "share_links/tag/detail.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["tag"] = Tag.objects.get(translations__slug=self.kwargs["slug"])
        return context

    def get_queryset(self):
        return Link.objects.filter(
            tags__translations__slug__contains=self.kwargs["slug"]
        ).order_by("-date_added")


class CategoryView(DetailView):
    model = Category
    paginate_by = PAGINATION_SIZE
    template_name = "share_links/category/detail.html"


class StatsView(TemplateView):
    template_name = "share_links/misc/stats.html"

    def get_stats(self):
        all_links = Link.objects.count()
        most_used_tag = {"nb": 0, "tag": None}
        for tag in Tag.objects.all().prefetch_related("links"):
            nb = tag.links.count()
            if nb > most_used_tag["nb"]:
                most_used_tag = {"nb": nb, "tag": tag}
        authors = {}
        for author in get_user_model().objects.all():
            authors[author.username] = len(Link.objects.filter(added_by=author))
        languages_list = list(
            Link.objects.values_list("language")
            .distinct()
            .annotate(nb=Count("language"))
            .order_by("-nb")
        )
        languages = {}
        languages_total = 0
        for language in languages_list:
            if language[0] is not None:
                languages[language[0]] = language[1]
                languages_total = languages_total + int(language[1])
        languages["not defined"] = all_links - languages_total
        return {
            "nb_links": all_links,
            "nb_comments": CommentwithCaptcha.objects.all().count(),
            "nb_tags": Tag.objects.count(),
            "most_used_tag": most_used_tag,
            "authors": authors,
            "languages": languages,
        }

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["stats"] = self.get_stats()
        return context


class WebRingView(TemplateView):
    template_name = "share_links/misc/webring.html"

    def get_webring(self):
        webring_json = load(
            urlopen("https://gitlab.com/sodimel/share-links/-/raw/main/webring.json")
        )
        return webring_json

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["webring"] = self.get_webring()

        own_instance = next(
            (
                index
                for (index, dictionnary) in enumerate(context["webring"])
                if self.request.site.domain in dictionnary["url"]
            ),
            None,
        )

        if own_instance is not None:
            context["previous"] = context["webring"][
                (own_instance - 1) % len(context["webring"])
            ]
            context["next"] = context["webring"][
                (own_instance + 1) % len(context["webring"])
            ]
        context["random"] = choice(context["webring"])
        return context


def get_title_view(request):
    data = {"status": "", "title": ""}
    if "url" in request.GET:
        status, text = get_title(request.GET["url"])
        if status is True:
            data["title"] = text
            data["status"] = "ok"
        else:
            data["title"] = ""
            data["status"] = text
    else:
        data["title"] = ""
        data["status"] = _("Error: no url supplied.")
    return JsonResponse(data)


def get_lang_view(request):
    data = {"status": "", "lang": ""}
    if "url" in request.GET:
        status, text = get_lang(request.GET["url"])
        if status is True:
            data["lang"] = text
            data["status"] = "ok"
        else:
            data["lang"] = ""
            data["status"] = text
    else:
        data["lang"] = ""
        data["status"] = _("Error: no url supplied.")
    return JsonResponse(data)


class AboutView(TemplateView):
    template_name = "share_links/misc/about.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["about_text"] = AboutContactPages.objects.first()
        return context


class ContactView(TemplateView):
    template_name = "share_links/misc/contact.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["contact_text"] = AboutContactPages.objects.last()
        return context


class RSSView(TemplateView):
    template_name = "share_links/rss/home.html"


def get_description(item, display_type=False):
    activate(get_language())

    long_description = strip_tags(item.description)
    description = " ".join(long_description.split(" ")[:20])
    if len(description) < len(long_description):
        description += "..."

    if display_type:
        if type(item) is Link:
            return f"(link) - {description}"
        return f"(tag) - {description}"
    return description


class RSSNewView(Feed):
    title = _("New links & tags")
    link = "/all/"
    description = _("Creation of links and tags.")

    def items(self):
        links = list(Link.objects.order_by("-id")[:5])
        tags = list(Tag.objects.order_by("-id")[:5])
        links_tags = links + tags
        links_tags.sort(key=lambda r: r.date_added, reverse=True)
        return links_tags[:5]

    def item_title(self, item):
        if type(item) is Link:
            if item.title:
                return escape(str(item.title))
            return escape(str(item.link))
        return escape(str(item.tag))

    def item_description(self, item):
        return get_description(item, display_type=True)

    def item_link(self, item):
        if type(item) is Link:
            return reverse("link", args=[item.pk])
        return reverse("tag", args=[item.slug])

    def item_pubdate(self, item):
        return item.date_added

    def item_unique_id(self, item):
        return item.pk

    def item_categories(self, item):
        if type(item) is Link:
            return "link"
        return "tag"


class RSSNewLinksView(Feed):
    title = _("New links")
    link = "/links/"
    description = _("Creation of links.")

    def items(self):
        return Link.objects.order_by("-id")[:5]

    def item_title(self, item):
        if item.title:
            return escape(str(item.title))
        return escape(str(item.link))

    def item_description(self, item):
        return get_description(item)

    def item_link(self, item):
        return reverse("link", args=[item.pk])

    def item_pubdate(self, item):
        return item.date_added

    def item_unique_id(self, item):
        return item.pk

    def item_categories(self, item):
        return "link"


class RSSNewTagsView(Feed):
    title = _("New tags")
    link = "/tags/"
    description = _("Creation of tags.")

    def items(self):
        return Tag.objects.order_by("-id")[:5]

    def item_title(self, item):
        return escape(str(item.tag))

    def item_description(self, item):
        return get_description(item)

    def item_link(self, item):
        return reverse("tag", args=[item.slug])

    def item_pubdate(self, item):
        return item.date_added

    def item_unique_id(self, item):
        return item.pk

    def item_categories(self, item):
        return "tag"


class CollectionsView(FilterView):
    model = Collection
    paginate_by = PAGINATION_SIZE
    template_name = "share_links/collection/list.html"
    filterset_class = CollectionFilter

    def get_queryset(self):
        if self.request.GET.get("sort", False):
            sort = self.request.GET.get("sort", False)
            if sort == "new":
                objects = Collection.objects.all().order_by("-id")
            if sort == "old":
                objects = Collection.objects.all().order_by("id")
            if sort == "highlight":
                objects = Collection.objects.filter(highlight=True).order_by("-id")
        else:
            objects = Collection.objects.all().order_by("-id")
        return objects


class CollectionView(ListView):
    model = Collection
    paginate_by = PAGINATION_SIZE
    template_name = "share_links/collection/detail.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["collection"] = Collection.objects.get(
            translations__slug=self.kwargs["slug"]
        )
        return context



from html.parser import HTMLParser
from re import search
from urllib.error import HTTPError, URLError
from urllib.parse import quote, urlparse, urlsplit, urlunsplit
from urllib.request import Request, urlopen
from zlib import MAX_WBITS, decompress

from django.conf.global_settings import LANGUAGES as FULL_LANGUAGES_LIST
from django.contrib.sites.models import Site
from django.utils.html import mark_safe
from django.utils.translation import gettext_lazy as _

from share_links.__version__ import version


def iri2uri(iri):
    """Thx https://stackoverflow.com/a/42309027/6813732 !"""
    uri = ""
    if isinstance(iri, str):
        (scheme, netloc, path, query, fragment) = urlsplit(iri)
        scheme = quote(scheme)
        netloc = netloc.encode("idna").decode("utf-8")
        path = quote(path)
        query = quote(query)
        fragment = quote(fragment)
        uri = urlunsplit((scheme, netloc, path, query, fragment))
    return uri


def get_full_url(url):
    parsed_url = urlparse(url)
    new_url = parsed_url.scheme + "://" + parsed_url.netloc + quote(parsed_url.path)
    if parsed_url.query:
        new_url += "?" + parsed_url.query
    new_url = iri2uri(new_url)
    return new_url


def get_request(url):
    request = Request(url)
    request.add_header("Accept-Encoding", "gzip, deflate")
    request.add_header(
        "Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8"
    )
    request.add_header(
        "User-agent",
        f"share-links v{version} (gitlab.com/sodimel/share-links) instance ({Site.objects.get_current().domain})",
    )
    return request


class TitleParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.match = False
        self.title = []

    def handle_starttag(self, tag, attributes):
        self.match = tag == "title"

    def handle_data(self, data):
        if self.match:
            self.title.append(data)
            self.match = False


def get_parsed_html(url, request):
    try:
        html = urlopen(request, timeout=10).read()
        if html[0:3] == b"\x1f\x8b\x08":  # why tf is this html content still gzipped?
            # TODO: check size of content, if too big, dont decompress (gzip bomb)
            html = decompress(html, 16 + MAX_WBITS)
        html = html.decode("utf-8")
    except UnicodeDecodeError:
        html = html.decode("iso-8859-1")
    except HTTPError as e:
        if e.code == 404:  # not found
            return (
                False,
                _(
                    "Got a HTTP status of 404, Not found. The page does not exist anymore."
                ),
            )
        if (
            e.code == 406
        ):  # not acceptable - should not happen anymore since we're using a header to say we accept html, but just in case
            return (
                False,
                _(
                    "Got a HTTP status of 406, Not Acceptable. It seems that this page does not really like scrapping, you're gonna take the data yourself."
                ),
            )
        if e.code == 410:  # gone
            return (
                False,
                _(
                    "Got a HTTP status of 410, Gone. The resource is likely not available anymore."
                ),
            )
        return (False, _(f"Unknown error, HTTP code {e.code}."))
    except URLError:
        return (False, _("Url is unreachable."))
    return (True, html)


def get_title(url):
    url = get_full_url(url)
    request = get_request(url)
    status, html = get_parsed_html(url, request)
    if status is False:
        return (status, html)

    parser = TitleParser()
    parser.feed(html)
    if len(parser.title):
        title = mark_safe(parser.title[0].strip())
    else:
        title = None

    if title:
        return (True, title)
    return (False, _("Got an empty title, you'll need to be creative!"))


def get_lang(url):
    url = get_full_url(url)
    request = get_request(url)
    status, html = get_parsed_html(url, request)
    if status is False:
        return (status, html)

    match = search(r"lang=['\"]?([a-z]+)[\"']?>?", html)
    if match:
        match = match.group(1)
        for language in FULL_LANGUAGES_LIST:
            if match == language[0]:
                return (True, match)
        return (
            False,
            _(
                "Lang attribute found does not match any of available value in LANGUAGE_CHOICES."
            ),
        )
    return (False, _("Cannot find a lang attribute."))



def get_model():
    from share_links.apps.share_links_comments.models import (  # noqa E402
        CommentwithCaptcha,
    )

    return CommentwithCaptcha


def get_form():
    from share_links.apps.share_links_comments.forms import (  # noqa E402
        CommentwithCaptchaForm,
    )

    return CommentwithCaptchaForm


from .apps import ShareLinksCommentsConfig  # noqa E402

default_app_config = ShareLinksCommentsConfig



# tests/test_filters.py

import pytest
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.test import RequestFactory
from django.utils import timezone

from share_links.filters import CollectionFilter, LinkFilter, TagsFilter
from share_links.models import Collection, Link, Tag


@pytest.mark.django_db
def test_link_filter_highlight():
    Link.objects.create(link="https://example1.com", highlight=True)
    Link.objects.create(link="https://example2.com", highlight=False)

    request = RequestFactory().get("/", {"highlight": "True"})
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = LinkFilter(request=request, queryset=Link.objects.all())

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2
    assert filtered_queryset.first().highlight is True


@pytest.mark.django_db
def test_link_filter_added_by():
    User = get_user_model()
    user1 = User.objects.create(username="user1")
    user2 = User.objects.create(username="user2")
    Link.objects.create(link="https://example1.com", added_by=user1)
    Link.objects.create(link="https://example2.com", added_by=user2)

    request = RequestFactory().get("/", {"added_by": user1.id})
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = LinkFilter(request=request, queryset=Link.objects.all())

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2
    assert filtered_queryset.first().added_by == user1


@pytest.mark.django_db
def test_link_filter_date_added():
    Link.objects.create(
        link="https://example1.com",
        date_added=timezone.now() - timezone.timedelta(days=1),
    )
    Link.objects.create(link="https://example2.com", date_added=timezone.now())

    request = RequestFactory().get(
        "/", {"date_added_after": timezone.now() - timezone.timedelta(days=2)}
    )
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = LinkFilter(request=request, queryset=Link.objects.all())

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2


@pytest.mark.django_db
def test_link_filter_language():
    Link.objects.create(link="https://example1.com", language="en")
    Link.objects.create(link="https://example2.com", language="fr")

    request = RequestFactory().get("/", {"language": "en"})
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = LinkFilter(request=request, queryset=Link.objects.all())

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2
    assert filtered_queryset.first().language == "en"


@pytest.mark.django_db
def test_link_filter_online():
    Link.objects.create(link="https://example1.com", online=True)
    Link.objects.create(link="https://example2.com", online=False)

    request = RequestFactory().get("/", {"online": "True"})
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = LinkFilter(request=request, queryset=Link.objects.all())

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2
    assert filtered_queryset.first().online is True


@pytest.mark.django_db
def test_link_filter_comments():
    site, created = Site.objects.get_or_create(id=1)

    link_with_comments = Link.objects.create(link="https://example1.com")
    link_with_comments.comments.create(comment="Test comment", site=site)
    Link.objects.create(link="https://example2.com")

    request = RequestFactory().get("/", {"comments": "True"})
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = LinkFilter(request=request, queryset=Link.objects.all())

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2
    assert filtered_queryset.first().comments.exists()


@pytest.mark.django_db
def test_collection_filter_highlight():
    Collection.objects.create(name="Collection1", highlight=True)
    Collection.objects.create(name="Collection2", highlight=False)

    request = RequestFactory().get("/", {"highlight": "True"})
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = CollectionFilter(
        request=request, queryset=Collection.objects.all()
    )

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2
    assert filtered_queryset.first().highlight is True


@pytest.mark.django_db
def test_collection_filter_added_by():
    User = get_user_model()
    user1 = User.objects.create(username="user1")
    user2 = User.objects.create(username="user2")
    Collection.objects.create(name="Collection1", added_by=user1)
    Collection.objects.create(name="Collection2", added_by=user2)

    request = RequestFactory().get("/", {"added_by": user1.id})
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = CollectionFilter(
        request=request, queryset=Collection.objects.all()
    )

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2
    assert filtered_queryset.first().added_by == user1


@pytest.mark.django_db
def test_collection_filter_date_added():
    Collection.objects.create(
        name="Collection1", date_added=timezone.now() - timezone.timedelta(days=1)
    )
    Collection.objects.create(name="Collection2", date_added=timezone.now())

    request = RequestFactory().get(
        "/", {"date_added_after": timezone.now() - timezone.timedelta(days=2)}
    )
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = CollectionFilter(
        request=request, queryset=Collection.objects.all()
    )

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2


@pytest.mark.django_db
def test_tags_filter_highlight():
    Tag.objects.create(tag="Tag1", highlight=True)
    Tag.objects.create(tag="Tag2", highlight=False)

    request = RequestFactory().get("/", {"highlight": "True"})
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = TagsFilter(request=request, queryset=Tag.objects.all())

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2
    assert filtered_queryset.first().highlight is True


@pytest.mark.django_db
def test_tags_filter_name():
    Tag.objects.create(tag="Tag1")
    Tag.objects.create(tag="Tag2")

    request = RequestFactory().get("/", {"name": "Tag1"})
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = TagsFilter(request=request, queryset=Tag.objects.all())

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2
    assert filtered_queryset.first().tag == "Tag1"


@pytest.mark.django_db
def test_tags_filter_date_added():
    Tag.objects.create(
        tag="Tag1", date_added=timezone.now() - timezone.timedelta(days=1)
    )
    Tag.objects.create(tag="Tag2", date_added=timezone.now())

    request = RequestFactory().get(
        "/", {"date_added_after": timezone.now() - timezone.timedelta(days=2)}
    )
    request.resolver_match = type("ResolverMatch", (object,), {"kwargs": {}})()

    filter_instance = TagsFilter(request=request, queryset=Tag.objects.all())

    filtered_queryset = filter_instance.qs

    assert filtered_queryset.count() == 2



from django.contrib.auth.decorators import login_required
from django.urls import path
from django.utils.translation import gettext_lazy as _
from django_comments.feeds import LatestCommentFeed

from . import views

urlpatterns = [
    # search
    path(_("search/"), views.SearchView.as_view(), name="search"),
    # rss
    path(_("rss/"), views.RSSView.as_view(), name="rss_index"),
    path(_("rss/new/all/"), views.RSSNewView(), name="rss_new"),
    path(_("rss/new/links/"), views.RSSNewLinksView(), name="rss_new_links"),
    path(_("rss/new/tags/"), views.RSSNewTagsView(), name="rss_new_tags"),
    path("rss/latest-comments/", LatestCommentFeed()),
    # misc
    path(_("about/"), views.AboutView.as_view(), name="about"),
    path(_("contact/"), views.ContactView.as_view(), name="contact"),
    path(_("statistics/"), views.StatsView.as_view(), name="stats"),
    path(_("webring/"), views.WebRingView.as_view(), name="webring"),
    # smol api
    path(_("get-title/"), login_required(views.get_title_view), name="get_title"),
    path(_("get-lang/"), login_required(views.get_lang_view), name="get_lang"),
    # tags
    path(_("tag/<str:slug>/"), views.TagView.as_view(), name="tag"),
    path(_("tags/"), views.TagsView.as_view(), name="tags"),
    path(_("category/<int:pk>/"), views.CategoryView.as_view(), name="category"),
    # Comments
    # path(_("last-comments/"), views.LastCommentsView.as_view(), name="comments"),
    # collections
    path(
        _("collection/<str:slug>/"), views.CollectionView.as_view(), name="collection"
    ),
    path(_("collections/"), views.CollectionsView.as_view(), name="collections"),
    # links
    path(_("link/<int:pk>/"), views.LinkView.as_view(), name="link"),
    path(
        _("domain-name/<str:domain_name>/"),
        views.LinkIndexView.as_view(),
        name="link_index",
    ),
    path(_("random/"), views.go_random_link, name="random"),
    path("", views.LinksView.as_view(), name="homepage"),
]



from django.contrib.admin import SimpleListFilter
from django.db.models import Count
from django.utils.translation import gettext as _


class OnlineOfflineFilter(SimpleListFilter):
    title = _("is online?")
    parameter_name = "online"

    def lookups(self, request, model_admin):
        return [
            ("online", _("Online")),
            ("offline", _("Offline")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "online":
            return queryset.filter(online=True)
        if self.value():
            return queryset.filter(online=False)


class HasAtLeastOneTagFilter(SimpleListFilter):
    title = _("has at least one tag")
    parameter_name = "one_tag"

    def lookups(self, request, model_admin):
        return [
            ("one_tag", _("At least one tag")),
            ("no_tag", _("No tag")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "one_tag":
            return queryset.annotate(num_tags=Count("tags")).filter(num_tags__gt=0)
        if self.value():
            return queryset.annotate(num_tags=Count("tags")).filter(num_tags__lt=1)


class RandomFilter(SimpleListFilter):
    title = _("random")
    parameter_name = "random"

    def lookups(self, request, model_admin):
        return [
            ("random", _("Random")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "random":
            return queryset.order_by("?")



# Generated by Django 3.2.7 on 2021-12-16 22:08

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="CommentwithCaptcha",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "object_pk",
                    models.CharField(
                        db_index=True, max_length=64, verbose_name="object ID"
                    ),
                ),
                (
                    "user_name",
                    models.CharField(
                        blank=True, max_length=50, verbose_name="user's name"
                    ),
                ),
                (
                    "user_email",
                    models.EmailField(
                        blank=True, max_length=254, verbose_name="user's email address"
                    ),
                ),
                ("user_url", models.URLField(blank=True, verbose_name="user's URL")),
                ("comment", models.TextField(max_length=3000, verbose_name="comment")),
                (
                    "submit_date",
                    models.DateTimeField(
                        db_index=True, default=None, verbose_name="date/time submitted"
                    ),
                ),
                (
                    "ip_address",
                    models.GenericIPAddressField(
                        blank=True,
                        null=True,
                        unpack_ipv4=True,
                        verbose_name="IP address",
                    ),
                ),
                (
                    "is_public",
                    models.BooleanField(
                        default=True,
                        help_text="Uncheck this box to make the comment effectively disappear from the site.",
                        verbose_name="is public",
                    ),
                ),
                (
                    "is_removed",
                    models.BooleanField(
                        db_index=True,
                        default=False,
                        help_text='Check this box if the comment is inappropriate. A "This comment has been removed" message will be displayed instead.',
                        verbose_name="is removed",
                    ),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="content_type_set_for_commentwithcaptcha",
                        to="contenttypes.contenttype",
                        verbose_name="content type",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="sites.site"
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="commentwithcaptcha_comments",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
            options={
                "verbose_name": "comment",
                "verbose_name_plural": "comments",
                "ordering": ("submit_date",),
                "permissions": [("can_moderate", "Can moderate comments")],
                "abstract": False,
            },
        ),
    ]



# tests/test_utils.py

import pytest

from share_links.utils import get_full_url, get_lang, get_request, get_title, iri2uri


@pytest.mark.django_db
def test_iri2uri():
    iri = "http://example.com/é"
    expected_uri = "http://example.com/%C3%A9"
    assert iri2uri(iri) == expected_uri


@pytest.mark.django_db
def test_get_full_url():
    url = "http://example.com/é"
    expected_full_url = "http://example.com/%25C3%25A9"
    assert get_full_url(url) == expected_full_url


@pytest.mark.django_db
def test_get_request():
    url = "http://example.com"
    request = get_request(url)
    assert request.get_header("User-agent").startswith("share-links v")


@pytest.mark.django_db
def test_get_title_success(monkeypatch):
    def mock_urlopen(*args, **kwargs):
        class MockResponse:
            def read(self):
                return b"<html><head><title>Test Title</title></head></html>"

        return MockResponse()

    monkeypatch.setattr("share_links.utils.urlopen", mock_urlopen)

    url = "http://example.com"
    status, title = get_title(url)
    assert status is True
    assert title == "Test Title"


@pytest.mark.django_db
def test_get_title_empty(monkeypatch):
    def mock_urlopen(*args, **kwargs):
        class MockResponse:
            def read(self):
                return b"<html><head></head></html>"

        return MockResponse()

    monkeypatch.setattr("share_links.utils.urlopen", mock_urlopen)

    url = "http://example.com"
    status, title = get_title(url)
    assert status is False
    assert title == "Got an empty title, you'll need to be creative!"


@pytest.mark.django_db
def test_get_lang_success(monkeypatch):
    def mock_urlopen(*args, **kwargs):
        class MockResponse:
            def read(self):
                return b"<html lang='en'><head></head></html>"

        return MockResponse()

    monkeypatch.setattr("share_links.utils.urlopen", mock_urlopen)

    url = "http://example.com"
    status, lang = get_lang(url)
    assert status is True
    assert lang == "en"


@pytest.mark.django_db
def test_get_lang_not_found(monkeypatch):
    def mock_urlopen(*args, **kwargs):
        class MockResponse:
            def read(self):
                return b"<html><head></head></html>"

        return MockResponse()

    monkeypatch.setattr("share_links.utils.urlopen", mock_urlopen)

    url = "http://example.com"
    status, lang = get_lang(url)
    assert status is False
    assert lang == "Cannot find a lang attribute."



from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse_lazy
from django.utils.html import format_html
from django.utils.translation import gettext as _

from .models import CommentwithCaptcha


@admin.register(CommentwithCaptcha)
class CommentwithCaptchaAdmin(admin.ModelAdmin):
    list_display = [
        "id",
        "submit_date",
        "user_name",
        "user_email",
        "user_url",
        "comment",
        "link_url",
        "is_public",
        "is_removed",
        "comment_url",
    ]
    list_display_links = ["submit_date"]
    date_hierarchy = "submit_date"
    list_editable = ["is_public", "is_removed"]
    readonly_fields = [
        "user_name",
        "user",
        "user_email",
        "submit_date",
        "ip_address",
        "link_url",
        "link_comments",
        "id",
        "site",
    ]
    list_select_related = True

    fieldsets = (
        (None, {"fields": ("id", "is_public", "is_removed", "comment", "user_url")}),
        (
            _("User infos"),
            {
                "fields": ("user_name", "user_email", "ip_address"),
            },
        ),
        (
            _("Link infos"),
            {
                "fields": ("link_url", "link_comments"),
            },
        ),
    )

    def link_url(self, obj):
        return format_html(
            '<a href="{}">{}</a>',
            reverse_lazy("link", None, (obj.object_pk,)),
            self.link_title(obj),
        )

    link_url.short_description = _("Link url")

    def link_comments(self, obj):
        return CommentwithCaptcha.objects.filter(object_pk=obj.object_pk).count()

    link_comments.short_description = _("Nb of link comments")

    def comment_url(self, obj):
        return format_html(
            '<a href="{}#c{}">{}</a>',
            reverse_lazy("link", None, (obj.object_pk,)),
            obj.pk,
            _("View"),
        )

    comment_url.short_description = _("Comment URL")

    def link_title(self, obj):
        try:
            if obj.content_object.title:  # if title exist (try/except) and is not None
                return obj.content_object.title
        except ObjectDoesNotExist:
            ...
        return obj.content_object.link

    def comment(self, obj):
        return obj.comment[:50] + "..." if len(obj.comment) > 50 else obj.comment



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ShareLinksCommentsConfig(AppConfig):
    name = "share_links.apps.share_links_comments"
    verbose_name = _("Share links comments")



from django.core.management.base import BaseCommand

from share_links.models import Link
from share_links.utils import get_lang


class Command(BaseCommand):
    help = "Loop on each saved link and try to add a language if it's not defined."

    def handle(self, *args, **options):
        nb = 0
        errors = []
        links = Link.objects.all()
        nb_links = len(links)
        for link in links:
            if link.allow_override_language:
                self.stdout.write(
                    f"[{str(nb + len(errors))}/{nb_links}] Adding language to {link.link} if missing..."
                )
                try:
                    status, text = get_lang(link.link)
                    if status is True:
                        self.stdout.write(
                            self.style.SUCCESS(f"  (New) language: {text}")
                        )
                        link.language = text
                        link.save()
                    else:
                        self.stdout.write(f"  Error: {text}.")
                        errors.append(link)
                    nb += 1
                    self.stdout.write("Done!")
                except Exception as e:
                    self.stdout.write(self.style.ERROR(f"Error: {e}"))
                    errors.append(link)
            else:
                f"[{str(nb + len(errors))}/{nb_links}] Do not add language for {link.link} (allow_override_language is False)..."
                nb += 1

        self.stdout.write("")
        self.stdout.write(self.style.SUCCESS(f"Added {nb} langs!"))

        self.stdout.write(self.style.ERROR(f"There was {str(len(errors))} errors:"))
        for error in errors:
            self.stdout.write(f"[{error.id}] {error.link}")



# Generated by Django 4.1.1 on 2023-01-03 18:33

import django.db.models.deletion
import parler.fields
import parler.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("share_links", "0007_link_language"),
    ]

    operations = [
        migrations.CreateModel(
            name="Collection",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date_added", models.DateTimeField(auto_now_add=True)),
                (
                    "highlight",
                    models.BooleanField(
                        blank=True,
                        default=False,
                        help_text='Collection will be displayed with a special color, and will appear in the "highlighted" sort option.',
                    ),
                ),
                (
                    "added_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="CollectionLink",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "collection",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="share_links.collection",
                    ),
                ),
                (
                    "link",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="share_links.link",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="CollectionTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="The name of the collection.",
                        max_length=1024,
                        verbose_name="Nom",
                    ),
                ),
                (
                    "slug",
                    models.SlugField(
                        editable=False,
                        max_length=2000,
                        unique=True,
                        verbose_name="Slug",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="You can use markdown here.",
                        null=True,
                        verbose_name="Description",
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="share_links.collection",
                    ),
                ),
            ],
            options={
                "verbose_name": "collection Translation",
                "db_table": "share_links_collection_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="CollectionLinkTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="You can use markdown here. If the description is empty, the link description will be used (if it exists).",
                        null=True,
                        verbose_name="Description",
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="share_links.collectionlink",
                    ),
                ),
            ],
            options={
                "verbose_name": "collection link Translation",
                "db_table": "share_links_collectionlink_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
    ]



# Generated by Django 3.2.12 on 2022-05-04 20:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("share_links", "0002_alter_link_file"),
    ]

    operations = [
        migrations.AddField(
            model_name="linktranslation",
            name="title",
            field=models.CharField(
                blank=True, max_length=2000, null=True, verbose_name="Title"
            ),
        ),
    ]



from urllib.parse import urlparse

from django import template
from django.conf.global_settings import LANGUAGES as FULL_LANGUAGES_LIST
from django.utils.translation import gettext_lazy as _

from share_links.__version__ import version
from share_links.conf import DISPLAY_FAVICONS, FAVICON_EXTENSION, FAVICON_SERVICE_URL

register = template.Library()


@register.filter()
def get_domain_name(url):
    if not url.startswith("http"):
        url = "https://" + url
    return urlparse(url).netloc


@register.filter()
def get_translated_language(lang):
    try:
        lang = dict(FULL_LANGUAGES_LIST)[lang]
        return _(f"This page is in {lang}.")
    except:  # noqa:E722
        return _(f"This page is in {lang}.")


@register.filter()
def get_links_nb(tag):
    return tag.links.count()


@register.simple_tag(takes_context=True)
def query_transform(context, **kwargs):
    # thx https://stackoverflow.com/a/56824200/6813732 !
    query = context["request"].GET.copy()
    for k, v in kwargs.items():
        query[k] = v
    return query.urlencode()


@register.filter(is_safe=True)
def get_version(context, **kwargs):
    return f"{context}{version}"


@register.filter(is_safe=True)
def show_favicon(domain_name):
    if DISPLAY_FAVICONS:
        favicon_service_url = FAVICON_SERVICE_URL
        favicon_extension = FAVICON_EXTENSION
        favicon_alt_title_text = _("Favicon of") + " " + domain_name
        return f'<img src="{favicon_service_url}{get_domain_name(domain_name)}{favicon_extension}" title="{favicon_alt_title_text}" class="favicon" />'
    return ""



# tests/test_admin_utils.py

import pytest
from django.contrib.admin import AdminSite
from django.test import RequestFactory

from share_links.admin_utils import (
    HasAtLeastOneTagFilter,
    OnlineOfflineFilter,
    RandomFilter,
)
from share_links.models import Link, Tag


@pytest.mark.django_db
def test_online_offline_filter():
    Link.objects.create(link="https://example1.com", online=True)
    Link.objects.create(link="https://example2.com", online=False)

    request = RequestFactory().get("/")
    queryset = Link.objects.all()

    filter_instance = OnlineOfflineFilter(request, {}, Link, AdminSite())
    filter_instance.value = lambda: "online"
    filtered_queryset = filter_instance.queryset(request, queryset)
    assert filtered_queryset.count() == 1
    assert filtered_queryset.first().online is True

    filter_instance.value = lambda: "offline"
    filtered_queryset = filter_instance.queryset(request, queryset)
    assert filtered_queryset.count() == 1
    assert filtered_queryset.first().online is False


@pytest.mark.django_db
def test_has_at_least_one_tag_filter():
    tag = Tag.objects.create(tag="Test Tag")
    link_with_tag = Link.objects.create(link="https://example1.com")
    link_with_tag.tags.add(tag)
    Link.objects.create(link="https://example2.com")

    request = RequestFactory().get("/")
    queryset = Link.objects.all()

    filter_instance = HasAtLeastOneTagFilter(request, {}, Link, AdminSite())
    filter_instance.value = lambda: "one_tag"
    filtered_queryset = filter_instance.queryset(request, queryset)
    assert filtered_queryset.count() == 1
    assert filtered_queryset.first().tags.count() > 0

    filter_instance.value = lambda: "no_tag"
    filtered_queryset = filter_instance.queryset(request, queryset)
    assert filtered_queryset.count() == 1
    assert filtered_queryset.first().tags.count() == 0


@pytest.mark.django_db
def test_random_filter():
    # Create some test data
    Link.objects.create(link="https://example1.com")
    Link.objects.create(link="https://example2.com")

    request = RequestFactory().get("/")
    queryset = Link.objects.all()

    filter_instance = RandomFilter(request, {}, Link, AdminSite())
    filter_instance.value = lambda: "random"
    filtered_queryset = filter_instance.queryset(request, queryset)
    assert filtered_queryset.count() == 2
    assert all([link in queryset for link in filtered_queryset])



from django.conf import settings

truthy_values = ("true", "True", "1", "yes", "Yes", "y", "Y")

if hasattr(settings, "SHARE_LINKS_USE_WEASYPRINT"):
    USE_WEASYPRINT = (
        True if settings.SHARE_LINKS_USE_WEASYPRINT in truthy_values else False
    )
else:
    USE_WEASYPRINT = False

if hasattr(settings, "SHARE_LINKS_SHOW_WEBARCHIVE_LINK"):
    SHOW_WEBARCHIVE_LINK = (
        True if settings.SHARE_LINKS_SHOW_WEBARCHIVE_LINK in truthy_values else False
    )
else:
    SHOW_WEBARCHIVE_LINK = True

if hasattr(settings, "SHARE_LINKS_DISPLAY_FAVICONS"):
    DISPLAY_FAVICONS = (
        True if settings.SHARE_LINKS_DISPLAY_FAVICONS in truthy_values else False
    )
else:
    DISPLAY_FAVICONS = False

if hasattr(settings, "SHARE_LINKS_FAVICON_SERVICE_URL"):
    FAVICON_SERVICE_URL = settings.SHARE_LINKS_FAVICON_SERVICE_URL
else:
    FAVICON_SERVICE_URL = "https://icons.duckduckgo.com/ip3/"

if hasattr(settings, "SHARE_LINKS_FAVICON_EXTENSION"):
    FAVICON_EXTENSION = settings.SHARE_LINKS_FAVICON_EXTENSION
else:
    FAVICON_EXTENSION = ".ico"

if hasattr(settings, "SHARE_LINKS_SEARCH_MIN_LENGTH"):
    SEARCH_MIN_LENGTH = int(settings.SHARE_LINKS_SEARCH_MIN_LENGTH)
else:
    SEARCH_MIN_LENGTH = 5

if hasattr(settings, "SHARE_LINKS_PAGINATION_SIZE"):
    PAGINATION_SIZE = int(settings.SHARE_LINKS_PAGINATION_SIZE)
else:
    PAGINATION_SIZE = 10



# Generated by Django 4.1.1 on 2023-06-10 14:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "share_links",
            "0008_collection_collectionlink_collectiontranslation_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="link",
            name="allow_override_language",
            field=models.BooleanField(
                default=False,
                help_text="Management commands will try to fetch the language from the lang attribute.",
                verbose_name="Allow override language?",
            ),
        ),
        migrations.AddField(
            model_name="link",
            name="allow_override_title",
            field=models.BooleanField(
                default=False,
                help_text="Management commands will try to fetch the title from the website <title> tag.",
                verbose_name="Allow override title?",
            ),
        ),
        migrations.AlterField(
            model_name="collectiontranslation",
            name="name",
            field=models.CharField(
                help_text="The name of the collection.",
                max_length=1024,
                verbose_name="Name",
            ),
        ),
    ]



from modeltranslation.translator import TranslationOptions, register

from .models import AboutContactPages, Link, Tag


@register(Link)
class LinkTranslationOptions(TranslationOptions):
    fields = ("description",)


@register(Tag)
class TagTranslationOptions(TranslationOptions):
    fields = (
        "tag",
        "description",
    )


@register(AboutContactPages)
class AboutContactPagesTranslationOptions(TranslationOptions):
    fields = (
        "page_title",
        "description",
    )



# Generated by Django 3.2.7 on 2021-12-16 22:13

import django.db.models.deletion
import parler.fields
import parler.models
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="AboutContactPages",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "About & Contact Pages",
                "verbose_name_plural": "About & Contact Pages",
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="HistoryLink",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("time", models.DateTimeField(auto_now_add=True)),
                ("link_id", models.SmallIntegerField(blank=True, null=True)),
                ("link", models.CharField(max_length=2000, null=True)),
                ("description", models.TextField(max_length=100000, null=True)),
                (
                    "type",
                    models.CharField(
                        blank=True,
                        choices=[("new", "N"), ("update", "U"), ("remove", "R")],
                        max_length=6,
                    ),
                ),
            ],
            options={
                "verbose_name": "History of Links",
                "verbose_name_plural": "History of Links",
            },
        ),
        migrations.CreateModel(
            name="HistoryTag",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("time", models.DateTimeField(auto_now_add=True)),
                ("tag_id", models.SmallIntegerField(blank=True, null=True)),
                ("tag", models.CharField(max_length=2000, null=True)),
                ("description", models.TextField(max_length=100000, null=True)),
                (
                    "type",
                    models.CharField(
                        blank=True,
                        choices=[("new", "N"), ("update", "U"), ("remove", "R")],
                        max_length=6,
                    ),
                ),
            ],
            options={
                "verbose_name": "History of Tags",
                "verbose_name_plural": "History of Tags",
            },
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "highlight",
                    models.BooleanField(
                        blank=True,
                        default=False,
                        help_text='Tag will be displayed with a special color, and will appear in the "highlighted" sort option.',
                    ),
                ),
                ("date_added", models.DateTimeField(auto_now_add=True)),
            ],
            options={
                "verbose_name": "Tag",
                "verbose_name_plural": "Tags",
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="Link",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "link",
                    models.CharField(max_length=2000, unique=True, verbose_name="Link"),
                ),
                (
                    "highlight",
                    models.BooleanField(
                        blank=True,
                        default=False,
                        help_text='Link will be displayed with a special color, and will appear in the "highlighted" sort option.',
                    ),
                ),
                ("date_added", models.DateTimeField(auto_now_add=True)),
                (
                    "file",
                    models.FileField(
                        blank=True,
                        help_text="Upload pdf or screenshot of the page. If this field is empty and USE_WEASYPRINT is set to True (currently set to False), then the pdf will be auto-generated using weasyprint.",
                        null=True,
                        upload_to="pdfs/",
                    ),
                ),
                (
                    "save_file",
                    models.BooleanField(
                        default=True,
                        help_text="Uncheck this box if you don't want the file to be auto-generated.",
                    ),
                ),
                (
                    "tags",
                    models.ManyToManyField(
                        blank=True, related_name="links", to="share_links.Tag"
                    ),
                ),
            ],
            options={
                "verbose_name": "Link",
                "verbose_name_plural": "Links",
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="TagTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                ("tag", models.CharField(max_length=2000, verbose_name="Tag")),
                (
                    "slug",
                    models.SlugField(
                        editable=False,
                        max_length=2000,
                        unique=True,
                        verbose_name="Slug",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="You can use markdown here.",
                        null=True,
                        verbose_name="Description",
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="share_links.tag",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tag Translation",
                "db_table": "share_links_tag_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="LinkTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="You can use markdown here.",
                        null=True,
                        verbose_name="Description",
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="share_links.link",
                    ),
                ),
            ],
            options={
                "verbose_name": "Link Translation",
                "db_table": "share_links_link_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="AboutContactPagesTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "page_title",
                    models.CharField(
                        blank=True, max_length=255, null=True, verbose_name="Title"
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="You can use markdown here.",
                        null=True,
                        verbose_name="Description",
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="share_links.aboutcontactpages",
                    ),
                ),
            ],
            options={
                "verbose_name": "About & Contact Pages Translation",
                "db_table": "share_links_aboutcontactpages_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
    ]



# Generated by Django 4.1.1 on 2022-11-27 22:20

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("share_links", "0005_link_added_by"),
    ]

    operations = [
        migrations.DeleteModel(
            name="HistoryLink",
        ),
        migrations.DeleteModel(
            name="HistoryTag",
        ),
    ]



from django_comments.forms import CommentForm
from simplemathcaptcha.fields import MathCaptchaField

from share_links.apps.share_links_comments.models import CommentwithCaptcha


class CommentwithCaptchaForm(CommentForm):
    captcha = MathCaptchaField()

    def get_comment_create_data(self, **kwargs):
        data = super().get_comment_create_data(**kwargs)
        return data

    def get_comment_model(self):
        if "captcha" in self.cleaned_data:
            del self.cleaned_data["captcha"]
        return CommentwithCaptcha



from django.dispatch import receiver
from django.utils.translation import gettext_lazy as _
from django_comments.abstracts import CommentAbstractModel
from django_comments.signals import comment_will_be_posted
from simplemathcaptcha.fields import MathCaptchaField


class CommentwithCaptcha(CommentAbstractModel):
    captcha = MathCaptchaField()

    class Meta:
        verbose_name = _("Moderate comments")
        verbose_name_plural = _("Moderate comments")


@receiver(comment_will_be_posted, sender=CommentwithCaptcha)
def handle_comments(sender, comment, request, **kwargs):
    comment.is_public = False



from django.conf import settings
from django.conf.global_settings import LANGUAGES as FULL_LANGUAGES_LIST
from django.contrib.contenttypes.fields import GenericRelation
from django.db import models
from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.urls import reverse
from django.utils import timezone
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from django_comments.moderation import CommentModerator, moderator
from parler.models import TranslatableModel, TranslatedFields, TranslationDoesNotExist

from share_links.apps.share_links_comments.models import CommentwithCaptcha
from share_links.conf import USE_WEASYPRINT


class Link(TranslatableModel):
    link = models.CharField(_("Link"), max_length=2000, unique=True)
    translations = TranslatedFields(
        title=models.CharField(_("Title"), max_length=2000, null=True, blank=True),
        description=models.TextField(
            _("Description"),
            blank=True,
            null=True,
            help_text=_("You can use markdown here."),
        ),
    )
    tags = models.ManyToManyField(to="Tag", related_name="links", blank=True)
    highlight = models.BooleanField(
        default=False,
        blank=True,
        help_text=_(
            'Link will be displayed with a special color, and will appear in the "highlighted" sort option.'
        ),
    )
    date_added = models.DateTimeField(auto_now_add=True, blank=True)
    file = models.FileField(
        upload_to="files/",
        null=True,
        blank=True,
        help_text=_(
            f"Upload pdf or screenshot of the page. If this field is empty and USE_WEASYPRINT is set to True (currently set to {USE_WEASYPRINT}), then the pdf will be auto-generated using weasyprint."
        ),
    )
    save_file = models.BooleanField(
        default=True,
        help_text=_(
            "Uncheck this box if you don't want the file to be auto-generated."
        ),
    )
    online = models.BooleanField(
        default=True,
        help_text=_(
            "Uncheck this box if the link is not online anymore (will provide the web archive link instead of the original link)."
        ),
    )
    added_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, blank=True, null=True
    )
    language = models.CharField(
        verbose_name=_("Language"),
        choices=FULL_LANGUAGES_LIST,
        max_length=7,
        null=True,
        blank=True,
    )
    allow_override_title = models.BooleanField(
        verbose_name=_("Allow override title?"),
        help_text=_(
            "Management commands will try to fetch the title from the website &lt;title&gt; tag."
        ),
        default=False,
    )
    allow_override_language = models.BooleanField(
        verbose_name=_("Allow override language?"),
        help_text=_(
            "Management commands will try to fetch the language from the lang attribute."
        ),
        default=False,
    )
    comments = GenericRelation(to=CommentwithCaptcha, object_id_field="object_pk")

    class Meta:
        verbose_name = _("Link")
        verbose_name_plural = _("Links")

    def __str__(self):
        try:
            return f"{self.title} ({self.link})"
        except TranslationDoesNotExist:
            return f"{self.link}"

    def get_absolute_url(self):
        return reverse("link", kwargs={"pk": self.pk})

    def save(self, *args, **kwargs):
        if USE_WEASYPRINT and not self.file and self.save_file:
            from re import compile

            from django.core.files.base import ContentFile
            from weasyprint import HTML

            url_format = compile(r"https?://(www\.)?")
            file_name = (
                url_format.sub("", self.link).strip().strip("/").replace("/", "_")
                + ".pdf"  # noqa
            )
            try:
                pdf = HTML(self.link).write_pdf()
                pdf = ContentFile(pdf)

                self.file.save(file_name, pdf, save=False)
            except Exception:  # so much random errors! ignore them for now
                pass

        super().save(*args, **kwargs)


class EntryModerator(CommentModerator):
    email_notification = False


moderator.register(Link, EntryModerator)


class Category(TranslatableModel):
    translations = TranslatedFields(
        label=models.CharField(_("Label"), max_length=2048),
        description=models.TextField(
            _("Description"),
            blank=True,
            null=True,
            help_text=_("You can use markdown here."),
        ),
    )

    class Meta:
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")

    def __str__(self):
        return self.label


class Tag(TranslatableModel):
    translations = TranslatedFields(
        tag=models.CharField(_("Tag"), max_length=2000),
        slug=models.SlugField(_("Slug"), max_length=2000, allow_unicode=True),
        description=models.TextField(
            _("Description"),
            blank=True,
            null=True,
            help_text=_("You can use markdown here."),
        ),
    )
    category = models.ForeignKey(
        to=Category,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="tags",
    )
    highlight = models.BooleanField(
        default=False,
        blank=True,
        help_text=_(
            'Tag will be displayed with a special color, and will appear in the "highlighted" sort option.'
        ),
    )
    date_added = models.DateTimeField(auto_now_add=True, blank=True)

    class Meta:
        verbose_name = _("Tag")
        verbose_name_plural = _("Tags")

    def __str__(self):
        return f"{self.tag}"

    def save(self, *args, **kwargs):
        self.slug = slugify(self.tag)
        super().save(*args, **kwargs)


@receiver(pre_save, sender=Tag)
def pre_save_for_categories_and_tags_fixture(sender, instance, **kwargs):
    if kwargs["raw"]:
        instance.date_added = timezone.now()


class AboutContactPages(TranslatableModel):
    translations = TranslatedFields(
        page_title=models.CharField(_("Title"), max_length=255, blank=True, null=True),
        description=models.TextField(
            _("Description"),
            blank=True,
            null=True,
            help_text=_("You can use markdown here."),
        ),
    )

    def __str__(self):
        return f"{str(self.id)} − {self.page_title}"

    class Meta:
        verbose_name = _("About & Contact Pages")
        verbose_name_plural = _("About & Contact Pages")


class Collection(TranslatableModel):
    translations = TranslatedFields(
        name=models.CharField(
            _("Name"),
            max_length=1024,
            help_text=_("The name of the collection."),
        ),
        slug=models.SlugField(_("Slug"), max_length=2000, editable=False, unique=True),
        description=models.TextField(
            _("Description"),
            blank=True,
            null=True,
            help_text=_("You can use markdown here."),
        ),
    )
    date_added = models.DateTimeField(auto_now_add=True, blank=True)
    added_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, blank=True, null=True
    )
    highlight = models.BooleanField(
        default=False,
        blank=True,
        help_text=_(
            'Collection will be displayed with a special color, and will appear in the "highlighted" sort option.'
        ),
    )

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        self.slug = slugify(self.name)
        super().save(*args, **kwargs)


class CollectionLink(TranslatableModel):
    translations = TranslatedFields(
        description=models.TextField(
            _("Description"),
            blank=True,
            null=True,
            help_text=_(
                "You can use markdown here. If the description is empty, the link description will be used (if it exists)."
            ),
        )
    )
    link = models.ForeignKey("Link", on_delete=models.CASCADE)
    collection = models.ForeignKey("Collection", on_delete=models.CASCADE)

    def __str__(self):
        if self.link.title:
            return self.link.title
        return self.link.link



from django import forms
from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.urls import path, reverse_lazy
from django.utils.html import format_html
from django.utils.translation import gettext as _
from parler.admin import TranslatableAdmin, TranslatableStackedInline

from .admin_utils import HasAtLeastOneTagFilter, OnlineOfflineFilter, RandomFilter
from .models import AboutContactPages, Category, Collection, CollectionLink, Link, Tag


@admin.register(AboutContactPages)
class AboutContactPagesAdmin(TranslatableAdmin):
    list_display = ("name",)

    def name(self, obj):
        if obj == AboutContactPages.objects.first():
            return _("About page")
        else:
            return _("Contact page")

    def message_user(self, *args):  # overridden method
        pass

    def save_delete_msg(self, request):
        messages.warning(
            request,
            _(
                "You can only create two \"AboutContactPages\" instances (first is About page, second is Contact page), and you can't delete any of them! If you know what you're doing, you can still add/delete objects using the model directly (not the admin like now)."
            ),
        )
        return False

    def delete_queryset(self, request, queryset):
        return self.save_delete_msg(request)

    def delete_model(self, request, client):
        return self.save_delete_msg(request)

    def save_model(self, request, obj, form, change):
        if AboutContactPages.objects.count() == 2 and not obj.id:
            return self.save_delete_msg(request)
        else:
            super(AboutContactPagesAdmin, self).save_model(request, obj, form, change)


@admin.register(Link)
class LinkAdmin(TranslatableAdmin):
    search_fields = ["id", "translations__title", "link"]

    list_select_related = True

    list_filter = (RandomFilter, OnlineOfflineFilter, HasAtLeastOneTagFilter)

    list_display = (
        "date_added",
        "highlight",
        "online",
        "link",
        "title_sortable",
        "view_link",
        "language",
        "added_by",
        "file_exist",
        "webarchive_url",
    )
    list_display_links = ("date_added",)

    list_editable = (
        "highlight",
        "online",
        "language",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "link",
                    ("online", "highlight"),
                    (
                        "title",
                        "language",
                    ),
                    "tags",
                    "description",
                )
            },
        ),
        (
            _("How to handle updates?"),
            {
                "fields": (
                    (
                        "allow_override_title",
                        "allow_override_language",
                    ),
                )
            },
        ),
        (
            _("Save file"),
            {
                "fields": (
                    (
                        "file",
                        "save_file",
                    ),
                )
            },
        ),
    )

    autocomplete_fields = [
        "tags",
    ]

    actions = ("add_tags",)

    class Media:
        js = ("share_links/fetch_title_of_url.js",)
        css = {"all": ("share_links/fetch_title_of_url.css",)}

    @admin.display(boolean=True)
    def file_exist(self, obj):
        return bool(obj.file)

    file_exist.short_description = _("File exist?")

    def title_sortable(self, obj):
        return obj.title

    title_sortable.short_description = _("Title")
    title_sortable.admin_order_field = "translations__title"

    def webarchive_url(self, obj):
        return format_html(
            '<a href="https://web.archive.org/save/{}" target="_blank">{}</a>',
            obj.link,
            _("Save this"),
        )

    webarchive_url.allow_tags = True
    webarchive_url.short_description = _("Web archive")

    def view_link(self, obj):
        return format_html(
            '<a href="{}">{}</a>',
            reverse_lazy("link", None, (obj.id,)),
            _("View"),
        )

    @admin.action(description=_("Add tags to current selection"))
    def add_tags(self, request, queryset):
        links_selected = queryset.values_list("pk", flat=True)
        return HttpResponseRedirect(
            "add-tags/?ids=%s" % (",".join(str(pk) for pk in links_selected),)
        )

    def save_model(self, request, obj, form, change):
        if not obj.id:
            obj.added_by = request.user
        super(LinkAdmin, self).save_model(request, obj, form, change)

    def get_urls(self):
        urls = super().get_urls()
        add_tags_url = [
            path("add-tags/", self.admin_site.admin_view(self.add_tags_view))
        ]
        return add_tags_url + urls

    def add_tags_view(self, request):
        links = Link.objects.filter(
            id__in=[int(id) for id in request.GET["ids"].split(",")]
        )
        if request.method == "POST":
            tags = Tag.objects.filter(
                id__in=[int(id) for id in request.POST.getlist("tags_select")]
            )
            for link in links:
                link.tags.add(*tags)
                link.save()

            self.message_user(
                request,
                _(
                    f"{len(links)} links where updated with the new tags {', '.join([str(tag) for tag in tags])}.",
                ),
                messages.SUCCESS,
            )
            return HttpResponseRedirect(
                reverse_lazy(
                    "admin:%s_%s_changelist"
                    % (self.model._meta.app_label, self.model._meta.model_name)
                )
            )
        else:
            tags = Tag.objects.all()

            class AddTagForm(forms.Form):
                tags_select = forms.ModelMultipleChoiceField(
                    widget=forms.CheckboxSelectMultiple, required=True, queryset=tags
                )

            form = AddTagForm()

            context = dict(
                self.admin_site.each_context(request), links=links, tags=tags, form=form
            )
            return TemplateResponse(request, "admin/add_tags.html", context)


@admin.register(Tag)
class TagAdmin(TranslatableAdmin):
    search_fields = ["translations__tag", "translations__slug"]

    autocomplete_fields = [
        "category",
    ]

    list_display = ("tag", "highlight", "link_nb", "slug")

    list_editable = ("highlight",)

    fieldsets = (
        (
            None,
            {
                "fields": ("tag", "slug", "category", "description", "highlight"),
            },
        ),
    )

    def link_nb(self, obj):
        return Link.objects.filter(tags=obj).count()

    link_nb.short_description = _("Link nb")


@admin.register(Category)
class CategoryAdmin(TranslatableAdmin):
    search_fields = ["label"]

    list_display = ("label", "tag_nb")

    def tag_nb(self, obj):
        return Tag.objects.filter(category=obj).count()

    tag_nb.short_description = _("Tag nb")


class CollectionLinkInline(TranslatableStackedInline, admin.StackedInline):
    model = CollectionLink
    autocomplete_fields = [
        "link",
    ]


@admin.register(Collection)
class CollectionAdmin(TranslatableAdmin):
    fields = [
        "name",
        "description",
        "highlight",
    ]

    inlines = [
        CollectionLinkInline,
    ]

    def save_model(self, request, obj, form, change):
        if not obj.id:
            obj.added_by = request.user
        super(CollectionAdmin, self).save_model(request, obj, form, change)



# tests/test_admin.py

import pytest
from django.contrib.admin import AdminSite
from django.contrib.auth import get_user_model
from django.contrib.messages.storage.fallback import FallbackStorage
from django.test import RequestFactory
from django.utils.translation import activate

from share_links.admin import (
    AboutContactPagesAdmin,
    CategoryAdmin,
    CollectionAdmin,
    LinkAdmin,
    TagAdmin,
)
from share_links.models import AboutContactPages, Category, Collection, Link, Tag


@pytest.mark.django_db
def test_about_contact_pages_admin_name():
    about_page = AboutContactPages.objects.create()
    contact_page = AboutContactPages.objects.create()

    request = RequestFactory().get("/")  # noqa: F841
    admin_site = AdminSite()
    admin = AboutContactPagesAdmin(AboutContactPages, admin_site)

    assert admin.name(about_page) == "About page"
    assert admin.name(contact_page) == "Contact page"


@pytest.mark.django_db
def test_about_contact_pages_admin_save_delete_msg():
    request = RequestFactory().get("/")
    request.user = "testuser"
    setattr(request, "session", "session")
    messages = FallbackStorage(request)
    setattr(request, "_messages", messages)

    admin_site = AdminSite()
    admin = AboutContactPagesAdmin(AboutContactPages, admin_site)

    assert admin.save_delete_msg(request) is False
    assert len(messages) == 1


@pytest.mark.django_db
def test_about_contact_pages_admin_save_model():
    request = RequestFactory().get("/")
    setattr(request, "session", "session")
    messages = FallbackStorage(request)
    setattr(request, "_messages", messages)

    admin_site = AdminSite()
    admin = AboutContactPagesAdmin(AboutContactPages, admin_site)

    obj = AboutContactPages()
    form = None
    change = False

    # Test saving when there are already two instances
    AboutContactPages.objects.create()
    AboutContactPages.objects.create()
    assert admin.save_model(request, obj, form, change) is False
    assert len(messages) == 1

    # Test saving when there are less than two instances
    AboutContactPages.objects.all().delete()
    admin.save_model(request, obj, form, change)
    assert AboutContactPages.objects.count() == 1


@pytest.mark.django_db
def test_link_admin_file_exist():
    link = Link.objects.create(link="https://example.com", file="example.pdf")

    request = RequestFactory().get("/")  # noqa: F841
    admin_site = AdminSite()
    admin = LinkAdmin(Link, admin_site)

    assert admin.file_exist(link) is True


@pytest.mark.django_db
def test_link_admin_title_sortable():
    link = Link.objects.create(link="https://example.com", title="Example Title")

    request = RequestFactory().get("/")  # noqa: F841
    admin_site = AdminSite()
    admin = LinkAdmin(Link, admin_site)

    assert admin.title_sortable(link) == "Example Title"


@pytest.mark.django_db
def test_link_admin_webarchive_url():
    link = Link.objects.create(link="https://example.com")

    request = RequestFactory().get("/")  # noqa: F841
    admin_site = AdminSite()
    admin = LinkAdmin(Link, admin_site)

    assert (
        admin.webarchive_url(link)
        == '<a href="https://web.archive.org/save/https://example.com" target="_blank">Save this</a>'
    )


@pytest.mark.django_db
def test_link_admin_view_link():
    link = Link.objects.create(link="https://example.com")

    request = RequestFactory().get("/")  # noqa: F841
    admin_site = AdminSite()
    admin = LinkAdmin(Link, admin_site)

    assert admin.view_link(link) == f'<a href="/en/link/{link.id}/">View</a>'


@pytest.mark.django_db
def test_tag_admin_link_nb():
    tag = Tag.objects.create(tag="Example Tag")
    link = Link.objects.create(link="https://example.com")
    link.tags.add(tag)

    request = RequestFactory().get("/")  # noqa: F841
    admin_site = AdminSite()
    admin = TagAdmin(Tag, admin_site)

    assert admin.link_nb(tag) == 1


@pytest.mark.django_db
def test_category_admin_tag_nb():
    category = Category.objects.create(label="Example Category")
    tag = Tag.objects.create(tag="Example Tag", category=category)  # noqa: F841

    request = RequestFactory().get("/")  # noqa: F841
    admin_site = AdminSite()
    admin = CategoryAdmin(Category, admin_site)

    assert admin.tag_nb(category) == 1


@pytest.mark.django_db
def test_collection_admin_save_model():
    activate("en")

    User = get_user_model()
    user = User.objects.create(username="testuser")

    request = RequestFactory().get("/")
    request.user = user
    setattr(request, "session", "session")
    messages = FallbackStorage(request)
    setattr(request, "_messages", messages)

    admin_site = AdminSite()
    admin = CollectionAdmin(Collection, admin_site)

    # Create a Collection object with a translation
    obj = Collection()
    obj.name = "Test Collection"
    obj.added_by = user
    obj.save()

    form = None
    change = False

    admin.save_model(request, obj, form, change)
    assert obj.added_by == user
    assert Collection.objects.count() == 1



# Generated by Django 3.2.12 on 2022-03-27 18:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("share_links", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="link",
            name="file",
            field=models.FileField(
                blank=True,
                help_text="Upload pdf or screenshot of the page. If this field is empty and USE_WEASYPRINT is set to True (currently set to False), then the pdf will be auto-generated using weasyprint.",
                null=True,
                upload_to="files/",
            ),
        ),
    ]



# Generated by Django 4.1.1 on 2022-12-20 12:54

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("contenttypes", "0002_remove_content_type_name"),
        ("share_links_comments", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="commentwithcaptcha",
            options={
                "verbose_name": "Moderate comments",
                "verbose_name_plural": "Moderate comments",
            },
        ),
        migrations.AlterField(
            model_name="commentwithcaptcha",
            name="content_type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="content_type_set_for_%(class)s",
                to="contenttypes.contenttype",
                verbose_name="content type",
            ),
        ),
        migrations.AlterField(
            model_name="commentwithcaptcha",
            name="user",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="%(class)s_comments",
                to=settings.AUTH_USER_MODEL,
                verbose_name="user",
            ),
        ),
    ]



from django.core.management.base import BaseCommand

from share_links.models import Link
from share_links.utils import get_title


class Command(BaseCommand):
    help = "Loop on each saved link and check if the link is still available."

    def handle(self, *args, **options):
        nb = 0
        errors = []
        links = Link.objects.all()
        nb_links = len(links)
        for link in links:
            print(link.link, end="...\n")
            try:
                status = get_title(link.link)[0]
                status = "V"
                if status is True:  # online
                    errors.append(link)
                    link.online = False
                    status = "X"
            except Exception:
                errors.append(link)
                link.online = False
                status = "X"
            link.save()
            self.stdout.write(f"[{status}] [{str(nb + len(errors))}/{nb_links}]")
            nb += 1

        self.stdout.write("")
        self.stdout.write(self.style.ERROR(f"There was {str(len(errors))} errors:"))
        for error in errors:
            self.stdout.write(f"[{error.id}] {error.link}")



# Generated by Django 3.2.13 on 2022-08-26 21:05

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("share_links", "0003_linktranslation_title"),
    ]

    operations = [
        migrations.AddField(
            model_name="link",
            name="online",
            field=models.BooleanField(
                default=True,
                help_text="Uncheck this box if the link is not online anymore (will provide the web archive link instead of the original link).",
            ),
        ),
    ]



import pytest

from share_links.models import Category, Collection, CollectionLink, Link, Tag


@pytest.mark.django_db
def test_link_creation():
    link = Link.objects.create(
        link="https://example.com",
        title="Example Title",
        description="Example Description",
        highlight=True,
        online=True,
        added_by=None,
        language="en",
        allow_override_title=True,
        allow_override_language=True,
    )
    assert link.link == "https://example.com"
    assert link.title == "Example Title"
    assert link.description == "Example Description"
    assert link.highlight is True
    assert link.online is True
    assert link.language == "en"
    assert link.allow_override_title is True
    assert link.allow_override_language is True


@pytest.mark.django_db
def test_link_save_with_weasyprint():
    link = Link.objects.create(
        link="https://example.com",
        title="Example Title",
        description="Example Description",
        highlight=True,
        online=True,
        added_by=None,
        language="en",
        allow_override_title=True,
        allow_override_language=True,
        save_file=True,
    )
    link.save()
    assert link.file is not None


@pytest.mark.django_db
def test_link_str_method():
    link = Link.objects.create(
        link="https://example.com",
        title="Example Title",
        description="Example Description",
    )
    assert str(link) == "Example Title (https://example.com)"

    link.title = None
    assert str(link) == "None (https://example.com)"


@pytest.mark.django_db
def test_category_creation():
    category = Category.objects.create(
        label="Example Category", description="Example Description"
    )
    assert category.label == "Example Category"
    assert category.description == "Example Description"


@pytest.mark.django_db
def test_category_str_method():
    category = Category.objects.create(
        label="Example Category", description="Example Description"
    )
    assert str(category) == "Example Category"


@pytest.mark.django_db
def test_tag_creation():
    tag = Tag.objects.create(
        tag="Example Tag", description="Example Description", highlight=True
    )
    assert tag.tag == "Example Tag"
    assert tag.description == "Example Description"
    assert tag.highlight is True


@pytest.mark.django_db
def test_tag_slug_creation():
    tag = Tag.objects.create(tag="Example Tag", description="Example Description")
    assert tag.slug == "example-tag"


@pytest.mark.django_db
def test_tag_str_method():
    tag = Tag.objects.create(tag="Example Tag", description="Example Description")
    assert str(tag) == "Example Tag"


@pytest.mark.django_db
def test_collection_creation():
    collection = Collection.objects.create(
        name="Example Collection", description="Example Description", highlight=True
    )
    assert collection.name == "Example Collection"
    assert collection.description == "Example Description"
    assert collection.highlight is True


@pytest.mark.django_db
def test_collection_slug_creation():
    collection = Collection.objects.create(
        name="Example Collection", description="Example Description"
    )
    assert collection.slug == "example-collection"


@pytest.mark.django_db
def test_collection_str_method():
    collection = Collection.objects.create(
        name="Example Collection", description="Example Description"
    )
    assert str(collection) == "Example Collection"


@pytest.mark.django_db
def test_collection_link_creation():
    link = Link.objects.create(
        link="https://example.com",
        title="Example Title",
        description="Example Description",
    )
    collection = Collection.objects.create(
        name="Example Collection", description="Example Description"
    )
    collection_link = CollectionLink.objects.create(
        link=link, collection=collection, description="Example Description"
    )
    assert collection_link.link == link
    assert collection_link.collection == collection
    assert collection_link.description == "Example Description"


@pytest.mark.django_db
def test_collection_link_str_method():
    link = Link.objects.create(
        link="https://example.com",
        title="Example Title",
        description="Example Description",
    )
    collection = Collection.objects.create(
        name="Example Collection", description="Example Description"
    )
    collection_link = CollectionLink.objects.create(
        link=link, collection=collection, description="Example Description"
    )
    assert str(collection_link) == "Example Title"
